/*
 * ChatListener.java
 *
 * Created on March 17, 2008, 3:20 PM
 */

package org.netkernel.jabber.endpoint;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;
import org.netkernel.layer0.nkf.*;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;


/**
 *  Listener for new conversations
 * @author  pjr
 */
public class ChatConversationListener implements ChatManagerListener, MessageListener
{	
	private JabberEndpoint mEndpoint;
	
	public ChatConversationListener(JabberEndpoint aEndpoint)
	{	mEndpoint=aEndpoint;
	}
	
	public void chatCreated(Chat chat, boolean param)
	{	if(!param)
		{	INKFRequestContext context=mEndpoint.getRequestContext();
			context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_INITIATED", chat.getParticipant());
			chat.addMessageListener(this);
		}
	}
	
	public void processMessage(Chat chat, Message message)
	{	XMPPError error=message.getError();
		if(error==null)
		{	INKFRequestContext context=mEndpoint.getRequestContext();
			context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_RECEIVED", message.getFrom());
			try
			{	ChatSpace space=new ChatSpace(chat);
				space.onCommissionSpace(context.getKernelContext().getKernel());
				INKFRequest req=context.createRequest("jabber:message");
				req.addArgument("from", message.getFrom());
				req.addArgument("to", message.getTo());
				req.addArgumentByValue("message",message);
				IRequestScopeLevel target=context.getKernelContext().getRequestScope();
				IRequestScopeLevel insert=RequestScopeLevelImpl.createDurableScopeLevel(space,target);
				req.setRequestScope(insert);
				context.issueRequest(req);
			}
			catch(Exception e)
			{	e.printStackTrace();
				context.logRaw(INKFRequestContext.LEVEL_WARNING, e.getMessage());
			}
		}
	}
}
