/*
 * JabberEndpoint.java
 *
 * Created on March 16, 2008, 8:29 AM
 */
package org.netkernel.jabber.endpoint;

import org.jivesoftware.smack.*;
import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.impl.EndpointMetaBuilder;
import org.netkernel.layer0.meta.impl.MetadataUtils;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.*;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.module.standard.endpoint.*;
import org.netkernel.urii.ISpace;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.ByteArrayInputStream;

/**
 *
 * @author  pjr
 */
public class JabberEndpoint extends StandardAccessorImpl implements ConnectionListener
{
	private NKFContextImpl mContext;
	private XMPPConnection mConnection;
	private boolean enabled= true;
	private ChatConversationListener mListener;
	private static final String ACCESSOR_GRAMMAR="<grammar><active><identifier>active:jabberSend</identifier><argument name='to'/><argument name='message'/></active></grammar>";
	
	/** Creates a new instance of JabberEndpoint */
	public JabberEndpoint()
	{	this.declareThreadSafe();
		
	}
	
	@Override
	protected void postCommission(INKFRequestContext context) throws Exception
	{
		context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_COMMISSION", (Object) null);
		Document cfg = context.source("param:config", Document.class);
		XMLReadable config = new XMLReadable(cfg.getDocumentElement());
		enabled = !("false".equals(config.getText("enabled")));
		if (enabled)
		{
			String server = config.getText("server");
			int port = config.getInt("port", 5222);
			String user = config.getText("user");
			String password = config.getText("password");
			ConnectionConfiguration xmppConfig = new ConnectionConfiguration(server, port);
			if (config.getText("tls").equals("true"))
			{
				xmppConfig.setSelfSignedCertificateEnabled(true);
				xmppConfig.setNotMatchingDomainCheckEnabled(true);
				xmppConfig.setSocketFactory(new DummySSLSocketFactory());
			}
			mConnection = new XMPPConnection(xmppConfig);
			try
			{
				mConnection.connect();
			}
			catch (Exception ex)
			{
				context.logFormatted(INKFRequestContext.LEVEL_WARNING, "ERR_JABBER_CONNECTED", server, ex.getMessage());
				throw ex;
			}
			context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_CONNECTED", server);
			try
			{
				mConnection.login(user, password, "NetKernel");
			}
			catch (Exception ex)
			{
				context.logFormatted(INKFRequestContext.LEVEL_WARNING, "ERR_JABBER_LOGGEDIN", user, ex.getMessage());
				throw ex;
			}
			context.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_LOGGEDIN", user);
			mConnection.addConnectionListener(this);
			mListener = new ChatConversationListener(this);
			mConnection.getChatManager().addChatListener(mListener);
		}
	}


	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	
		XMLReadable xr=new XMLReadable(new ByteArrayInputStream(ACCESSOR_GRAMMAR.getBytes()));
		Element e=(Element)((Document)xr.getContextNode()).getDocumentElement();
		EndpointMetaBuilder metabuilder=new EndpointMetaBuilder();
		MetadataUtils.parseGrammar(e, metabuilder, aKernel.getLogger());
		this.declareGrammar(metabuilder.getMeta(aKernel.getLogger()).getIdentifierGrammar());
		
		super.onCommissionEndpoint(aKernel, aSpace);
		mContext = new NKFTransportContextImpl(aKernel,aSpace,this);		
	}
	
	public void onDecommissionEndpoint() throws Exception
	{	try
		{	if (enabled) mConnection.disconnect();
			mContext=null;
			mConnection=null;
		}
		finally
		{	super.onDecommissionEndpoint();
		}
	}
	
	public void onSource(INKFRequestContext context) throws Exception
	{	if (enabled) {
			String to=context.getThisRequest().getArgumentValue("to");			
			Chat c=mConnection.getChatManager().createChat(to, mListener);
			String message=(String)context.source("arg:message", String.class);
			c.sendMessage(message);
		} else {
			throw new NKFException("JABBER_NOT_ENABLED", "XMPP message could not be sent as Jabber has been disabled");
		}
	}
	
	protected INKFRequestContext getRequestContext()
	{	return mContext;
	}

	//ConnectionListener Interface
	public void connectionClosed()
	{	String message=null;
		if(mConnection!=null)
		{	message=mConnection.getServiceName();			
		}
		mContext.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_CONN1", message);
	}

	public void connectionClosedOnError(Exception e)
	{	String message=null;
		if(mConnection!=null)
		{	message=mConnection.getServiceName();			
		}
		mContext.logFormatted(INKFRequestContext.LEVEL_WARNING, "MSG_JABBER_CONN2",  message, e.getMessage());
	}

	public void reconnectingIn(int time)
	{	mContext.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_CONN3", time);
	}

	public void reconnectionFailed(Exception e)
	{	mContext.logFormatted(INKFRequestContext.LEVEL_WARNING, "MSG_JABBER_CONN4",  e.getMessage());
	}

	public void reconnectionSuccessful()
	{	String message=null;
		if(mConnection!=null)
		{	message=mConnection.getServiceName();			
		}
		mContext.logFormatted(INKFRequestContext.LEVEL_INFO, "MSG_JABBER_CONN5", message);
	}
}